@@ -29,6 +29,8 @@
 #include "gconf-key-editor.h"
 #include "gconf-stock-icons.h"
 #include "gconf-util.h"
+#include "gedit-output-window.h"
+#include "gconf-search-dialog.h"
 #include <gconf/gconf.h>
 #include <libintl.h>
 #include <string.h>
@@ -182,6 +184,22 @@ gconf_editor_window_edit_bookmarks (GtkAction *action, GtkWidget *callback_data)
 	gtk_widget_show (dialog);
 }
 
+
+static void
+gconf_editor_window_search (GtkAction *action, GtkWidget *callback_data)
+{
+	static GtkWidget *dialog = NULL;
+
+	if (dialog != NULL) {
+		gtk_window_present (GTK_WINDOW (dialog));
+		return;
+	}
+
+	dialog = gconf_search_dialog_new (GTK_WINDOW (callback_data));
+	g_object_add_weak_pointer (G_OBJECT (dialog), (gpointer)&dialog);
+	gtk_widget_show (dialog);
+}
+
 static void
 gconf_editor_window_new_window (GtkAction *action, GtkWidget *callback_data)
 {
@@ -222,6 +240,22 @@ help_cb (GtkAction *action, GtkWidget *callback_data)
         }
 }
 
+static void
+gconf_editor_window_close_output_window (GtkWidget *widget, gpointer user_data)
+{
+	gedit_output_window_clear (GEDIT_OUTPUT_WINDOW (widget));
+	gtk_widget_hide (widget);
+}
+
+static void
+gconf_editor_window_output_window_changed (GtkWidget *widget, gchar *key, gpointer user_data)
+{
+	GConfEditorWindow *window = (GConfEditorWindow*) user_data;
+	gconf_editor_window_go_to (window, key);
+}
+
+
+
 
 static void
 gconf_editor_window_about_window (GtkAction *action, GtkWidget *callback_data)
@@ -588,6 +622,7 @@ gconf_editor_popup_window_set_as_mandatory (GtkAction *action, GtkWidget *callba
 static GtkActionEntry entries[] = {
 	{ "FileMenu", NULL, N_("_File"), NULL, NULL, NULL },
 	{ "EditMenu", NULL, N_("_Edit"), NULL, NULL, NULL },
+	{ "SearchMenu", NULL, N_("_Search"), NULL, NULL, NULL },
 	{ "BookmarksMenu", NULL, N_("_Bookmarks"), NULL, NULL, NULL },
 	{ "HelpMenu", NULL, N_("_Help"), NULL, NULL, NULL },
 
@@ -607,7 +642,9 @@ static GtkActionEntry entries[] = {
 	
 	{ "CopyKeyName", GTK_STOCK_COPY, N_("_Copy key name"), "<control>C", N_("Copy the name of the selected key"), 
 	  G_CALLBACK (gconf_editor_window_copy_key_name) },
-	  
+	{ "Find", GTK_STOCK_FIND, N_("Find"), "<control>F", N_("Find patterns in keys and values"), 
+	  G_CALLBACK (gconf_editor_window_search) },
+
 	{ "AddBookmark", NULL, N_("_Add bookmark"), NULL, N_("Add a bookmark to the selected directory"), 
 	  G_CALLBACK (gconf_editor_window_add_bookmark) },
 	{ "EditBookmarks", NULL, N_("_Edit bookmarks"), NULL, N_("Edit the bookmarks"), 
@@ -643,6 +680,7 @@ static const char *ui_description =
 	"		</menu>"
 	"		<menu action='EditMenu'>"
 	"			<menuitem action='CopyKeyName'/>"
+	"			<menuitem action='Find'/>"
 	"		</menu>"	
 	"		<menu action='BookmarksMenu'>"
 	"			<menuitem action='AddBookmark'/>"
@@ -673,6 +711,7 @@ static gchar *image_menu_items_paths[] = {
 	"/GConfEditorMenu/FileMenu/QuitGConfEditor",
 
 	"/GConfEditorMenu/EditMenu/CopyKeyName",
+	"/GConfEditorMenu/EditMenu/Find",
 
 	"/GConfEditorMenu/HelpMenu/HelpContents",
 	"/GConfEditorMenu/HelpMenu/AboutAction",
@@ -1165,12 +1204,13 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	GError *error;
 	
 	gtk_window_set_title (GTK_WINDOW (window), _("GConf editor"));
-	gtk_window_set_default_size (GTK_WINDOW (window), 640, 400);
+	gtk_window_set_default_size (GTK_WINDOW (window), 700, 550);
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (window), vbox);
 
 	action_group = gtk_action_group_new ("GConfEditorMenuActions");
+	gtk_action_group_set_translation_domain (action_group, NULL);
 	gtk_action_group_add_actions (action_group, entries, G_N_ELEMENTS (entries), window);
 	
 	window->ui_manager = gtk_ui_manager_new ();
@@ -1214,9 +1254,20 @@ gconf_editor_window_init (GConfEditorWindow *window)
 
 	/* Create content area */
 	hpaned = gtk_hpaned_new ();
-	gtk_paned_set_position (GTK_PANED (hpaned), 200);
+	gtk_paned_set_position (GTK_PANED (hpaned), 250);
 	gtk_box_pack_start (GTK_BOX (vbox), hpaned, TRUE, TRUE, 0);
 
+	/* Create ouput window */
+	window->output_window = gedit_output_window_new ();
+	gedit_output_window_set_select_multiple (GEDIT_OUTPUT_WINDOW (window->output_window),
+						 GTK_SELECTION_SINGLE);
+	gtk_box_pack_start (GTK_BOX (vbox), window->output_window, FALSE, FALSE, 0);
+	g_signal_connect (G_OBJECT (window->output_window), "close_requested",
+			  G_CALLBACK (gconf_editor_window_close_output_window), window);
+
+	g_signal_connect (G_OBJECT (window->output_window), "selection_changed",
+			  G_CALLBACK (gconf_editor_window_output_window_changed), window);
+	
 	/* Create status bar */
 	window->statusbar = gtk_statusbar_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), window->statusbar, FALSE, FALSE, 0);
@@ -1448,6 +1499,7 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	gtk_widget_show_all (vbox);
 	gtk_widget_hide (window->non_writable_label);
 	gtk_widget_hide (window->no_schema_label);
+	gtk_widget_hide (window->output_window);
 	
 	/* Set window icon */
 	icon = gdk_pixbuf_new_from_file (IMAGEDIR"/gconf-editor.png", NULL);
